package xxmath.frame;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import xxmath.hundred.Hundred;
import xxmath.hundred.HundredBuilder;
import xxmath.hundred.HundredDialog;
import xxmath.util.Config;

/**
 * [U[삷郁C̃EBhENXB
 * 
 * @author Ryo IKEDA
 */
public class MainFrame extends JFrame {

    /** \tgAo[W */
    private static final String XXMath = "XXMath 1.1";

    /** CSVt@Cpt@CtB^[ */
    private static final FileFilter fileFilter = new CSVFileFilter();

    /** ݒt@CǂݏIuWFNg */
    private final Config config = new Config("XXMath.xml");

    /** bNAhtB[ݒp */
    private String lookAndFeel = UIManager.getSystemLookAndFeelClassName();

    /** t@C_CAOōŌɊJfBNg */
    private File lastDir = new File(System.getProperty("user.dir"));

    /** R|[lgQ */
    private JPanel jContentPane;
    private JPanel pnButton;
    private JButton btStart;
    private JButton btCheck;
    private JButton btClear;
    private MainPanel mainPanel;
    private JPanel pnTimeScore;
    private TimePanel timePanel;
    private ScorePanel scorePanel;
    private JSlider jsFontSize;
    private JPanel pnFont;
    private JMenuBar mnBar;
    private JMenu mnFile;
    private JMenu mnEdit;
    private JMenu mnView;
    private JMenu mnLook;
    private JMenu mnHelp;
    private JMenuItem miNew;
    private JMenuItem miOption;
    private JMenuItem miOpen;
    private JMenuItem miSave;
    private JMenuItem miVersion;
    private JMenuItem miClear;
    private JMenuItem miStart;
    private JMenuItem miCheck;
    private JMenuItem miExit;

    public MainFrame() {
        super();
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        initialize();
    }

    private void initialize() {
        setJMenuBar(getMnBar());
        setContentPane(getJContentPane());
        setBounds(0, 0, 400, 400);
        setTitle(XXMath);
        setLocationRelativeTo(null);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                getMiExit().doClick();
            }
        });

        try {
            config.loadXML();

            int x = config.getInteger("window.x");
            int y = config.getInteger("window.y");
            int width = config.getInteger("window.width");
            int height = config.getInteger("window.height");
            int interval = config.getInteger("move.interval");
            int mode = config.getInteger("move.mode");
            Object moveMode = null;
            switch (mode) {
            case 0:
                moveMode = getMainPanel().MOVE_NONE;
                break;
            case 1:
                moveMode = getMainPanel().MOVE_RIGHT;
                break;
            case 2:
                moveMode = getMainPanel().MOVE_DOWN;
                break;
            case 3:
                moveMode = getMainPanel().MOVE_RANDOM;
                break;
            }
            int bordercolor = config.getInteger("mass.bordercolor");
            int correctcolor = config.getInteger("mass.correctcolor");
            int wrongcolor = config.getInteger("mass.wrongcolor");
            int defaultcolor = config.getInteger("mass.defaultcolor");
            int focuscolor = config.getInteger("mass.focuscolor");
            int focuscolor2 = config.getInteger("mass.focuscolor2");
            int fontsize = config.getInteger("mass.fontsize");
            String lastdir = config.get("window.lastdir");
            lookAndFeel = config.get("window.lookandfeel");

            setBounds(x, y, width, height);
            getMainPanel().setMoveInterval(interval);
            getMainPanel().setMove(moveMode);
            getMainPanel().getMassColor().setFocusBorderColor(new Color(bordercolor));
            getMainPanel().getMassColor().setCorrectBackColor(new Color(correctcolor));
            getMainPanel().getMassColor().setWrongBackColor(new Color(wrongcolor));
            getMainPanel().getMassColor().setDefaultBackColor(new Color(defaultcolor));
            getMainPanel().getMassColor().setFocusBackColor(new Color(focuscolor));
            getMainPanel().getMassColor().setFocusBackColor2(new Color(focuscolor2));
            getJsFontSize().setValue(fontsize);
            setLastDirector(lastdir);

            getMiClear().doClick();
        } catch (FileNotFoundException e) {
            // ǂݍݎ͖
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JMenu menu = getMnLook();
            for (int i = 0; i < menu.getItemCount(); i++) {
                JRadioButtonMenuItem mi = (JRadioButtonMenuItem)menu.getItem(i);
                String name = getLookAndFeelClassName(mi.getText());
                if (lookAndFeel.equals(name)) {
                    mi.doClick();
                }
            }
        }
    }

    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel(new BorderLayout());
            jContentPane.add(getPnFont(), BorderLayout.WEST);
            jContentPane.add(getPnTimeScore(), BorderLayout.NORTH);
            jContentPane.add(getPnButton(), BorderLayout.SOUTH);
            jContentPane.add(getMainPanel(), BorderLayout.CENTER);
        }
        return jContentPane;
    }

    private JPanel getPnTimeScore() {
        if (pnTimeScore == null) {
            pnTimeScore = new JPanel();
            pnTimeScore.setLayout(new GridLayout(1, 2));
            pnTimeScore.add(getTimePanel());
            pnTimeScore.add(getScorePanel());
        }
        return pnTimeScore;
    }

    private TimePanel getTimePanel() {
        if (timePanel == null) {
            timePanel = new TimePanel();
            timePanel.setBorder(BorderFactory.createEtchedBorder());
        }
        return timePanel;
    }

    private ScorePanel getScorePanel() {
        if (scorePanel == null) {
            scorePanel = new ScorePanel(getMainPanel());
            scorePanel.setBorder(BorderFactory.createEtchedBorder());
        }
        return scorePanel;
    }

    private MainPanel getMainPanel() {
        if (mainPanel == null) {
            mainPanel = new MainPanel();
        }
        return mainPanel;
    }

    private JPanel getPnFont() {
        if (pnFont == null) {
            pnFont = new JPanel();
            pnFont.setLayout(new GridLayout());
            pnFont.add(getJsFontSize());
        }
        return pnFont;
    }

    private JPanel getPnButton() {
        if (pnButton == null) {
            pnButton = new JPanel();
            pnButton.add(getBtStart());
            pnButton.add(getBtCheck());
            pnButton.add(getBtClear());
        }
        return pnButton;
    }

    private JButton getBtStart() {
        if (btStart == null) {
            btStart = new JButton("Jn");
            btStart.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    miStart.doClick();
                }
            });
        }
        return btStart;
    }

    private JButton getBtCheck() {
        if (btCheck == null) {
            btCheck = new JButton("̓_");
            btCheck.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    miCheck.doClick();
                }
            });
        }
        return btCheck;
    }

    private JButton getBtClear() {
        if (btClear == null) {
            btClear = new JButton("");
            btClear.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    miClear.doClick();
                }
            });
        }
        return btClear;
    }

    private JSlider getJsFontSize() {
        if (jsFontSize == null) {
            jsFontSize = new JSlider(12, 44, 16);
            jsFontSize.setOrientation(JSlider.VERTICAL);
            jsFontSize.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    getMainPanel().setFontSize(getJsFontSize().getValue());
                }
            });
        }
        return jsFontSize;
    }

    private JMenuBar getMnBar() {
        if (mnBar == null) {
            mnBar = new JMenuBar();
            mnBar.add(getMnFile());
            mnBar.add(getMnEdit());
            mnBar.add(getMnView());
            mnBar.add(getMnLook());
            mnBar.add(getMnHelp());
        }
        return mnBar;
    }

    private JMenu getMnFile() {
        if (mnFile == null) {
            mnFile = new JMenu("t@C");
            mnFile.add(getMiNew());
            mnFile.add(getMiOpen());
            mnFile.add(getMiSave());
            mnFile.add(getMiExit());
        }
        return mnFile;
    }

    private JMenu getMnEdit() {
        if (mnEdit == null) {
            mnEdit = new JMenu("ҏW");
            mnEdit.add(getMiStart());
            mnEdit.add(getMiCheck());
            mnEdit.add(getMiClear());
        }
        return mnEdit;
    }

    private JMenu getMnView() {
        if (mnView == null) {
            mnView = new JMenu("\");
            mnView.add(getMiOption());
        }
        return mnView;
    }

    private JMenu getMnLook() {
        if (mnLook == null) {
            mnLook = new JMenu("bN&tB[");
            ButtonGroup bg = new ButtonGroup();
            ActionListener actionListener = new LookAndFeelActionListener();
            
            /* pł郋bNAhtB[̈ꗗ擾A
             * ꂼ̖OŃWI{^j[ACe𐶐A
             * j[ƃ{^O[vɒǉB
             */
            UIManager.LookAndFeelInfo[] info =
                        UIManager.getInstalledLookAndFeels();
            for (UIManager.LookAndFeelInfo i : info) {
                JRadioButtonMenuItem mi = new JRadioButtonMenuItem(i.getName());
                mi.addActionListener(actionListener);
                bg.add(mi);
                mnLook.add(mi);
                
                // ݂̃bNAhtB[̍ڂIԂɂ
                String laf = UIManager.getLookAndFeel().getName();
                if (i.getName().equals(laf)) {
                    mi.setSelected(true);
                }
            }
        }
        return mnLook;
    }

    /**
     * ̕ƓÕbNAhtB[
     * pbP[W܂߂OŕԂBȂꍇAVXeftHg
     * bNAhtB[ԂB
     * 
     * FWindows
     *      -> com.sun.java.swing.plaf.windows.WindowsLookAndFeel
     * 
     * @param text ZbNAhtB[̖O
     * @return bNAhtB[̖O
     */
    private String getLookAndFeelClassName(String text) {
        UIManager.LookAndFeelInfo[] info =
            UIManager.getInstalledLookAndFeels();
        for (UIManager.LookAndFeelInfo i : info) {
            if (text.equals(i.getName())) {
                return i.getClassName();
            }
        }
        return UIManager.getSystemLookAndFeelClassName();
    }

    private JMenu getMnHelp() {
        if (mnHelp == null) {
            mnHelp = new JMenu("wv");
            mnHelp.add(getMiVersion());
        }
        return mnHelp;
    }

    private JMenuItem getMiNew() {
        if (miNew == null) {
            miNew = new JMenuItem("VK");
            miNew.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Hundred temp =
                        HundredDialog.showDialog(MainFrame.this, "VK쐬");
                    if (temp != null) {
                        getMainPanel().setHundred(temp);
                        getScorePanel().score();
                        getTimePanel().clear();
                    }
                }
            });
        }
        return miNew;
    }

    private JMenuItem getMiOpen() {
        if (miOpen == null) {
            miOpen = new JMenuItem("J");
            miOpen.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFileChooser fc = new JFileChooser(lastDir);
                    fc.addChoosableFileFilter(fileFilter);
                    fc.setDialogType(JFileChooser.OPEN_DIALOG);

                    int res = fc.showDialog(MainFrame.this, null);
                    lastDir = fc.getCurrentDirectory();
                    if (res != JFileChooser.APPROVE_OPTION) {
                        return;
                    }

                    try {
                        Hundred temp = HundredBuilder.read(fc.getSelectedFile());
                        getMainPanel().setHundred(temp);
                        getScorePanel().score();
                        getTimePanel().clear();
                    } catch (Exception error) {
                        JOptionPane.showMessageDialog(
                                MainFrame.this, error.getMessage(),
                                "G[", JOptionPane.ERROR_MESSAGE);
                        error.printStackTrace();
                    }
                }
            });
        }
        return miOpen;
    }

    private JMenuItem getMiSave() {
        if (miSave == null) {
            miSave = new JMenuItem("ۑ");
            miSave.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFileChooser fc = new JFileChooser(lastDir);
                    fc.addChoosableFileFilter(fileFilter);
                    fc.setDialogType(JFileChooser.SAVE_DIALOG);

                    int res = fc.showDialog(MainFrame.this, null);
                    lastDir = fc.getCurrentDirectory();
                    if (res != JFileChooser.APPROVE_OPTION) {
                        return;
                    }

                    File file = fc.getSelectedFile();
                    if (fc.getFileFilter().equals(fileFilter)
                            && !file.getName().endsWith(".csv")) {
                        file = new File(file.getAbsoluteFile() + ".csv");
                    }

                    try {
                        HundredBuilder.write(file, getMainPanel().getHundred());
                    } catch (Exception error) {
                        JOptionPane.showMessageDialog(
                            MainFrame.this, error.getMessage(),
                            "G[", JOptionPane.ERROR_MESSAGE);
                        error.printStackTrace();
                    }
                }
            });
        }
        return miSave;
    }

    private JMenuItem getMiExit() {
        if (miExit == null) {
            miExit = new JMenuItem("I");
            miExit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        config.set("window.lookandfeel", lookAndFeel);
                        config.setInteger("window.x", getX());
                        config.setInteger("window.y", getY());
                        config.setInteger("window.width", getWidth());
                        config.setInteger("window.height", getHeight());
                        config.setInteger("move.interval",
                                getMainPanel().getMoveInterval());
                        Object o = getMainPanel().getMove();
                        if (o.equals(getMainPanel().MOVE_NONE)) {
                            config.setInteger("move.mode", 0);
                        } else if (o.equals(getMainPanel().MOVE_RIGHT)) {
                            config.setInteger("move.mode", 1);
                        } else if (o.equals(getMainPanel().MOVE_DOWN)) {
                            config.setInteger("move.mode", 2);
                        } else if (o.equals(getMainPanel().MOVE_RANDOM)) {
                            config.setInteger("move.mode", 3);
                        }
                        config.setInteger("mass.bordercolor",
                                getMainPanel().getMassColor().getFocusBorderColor().getRGB());
                        config.setInteger("mass.correctcolor",
                                getMainPanel().getMassColor().getCorrectBackColor().getRGB());
                        config.setInteger("mass.wrongcolor",
                                getMainPanel().getMassColor().getWrongBackColor().getRGB());
                        config.setInteger("mass.defaultcolor",
                                getMainPanel().getMassColor().getDefaultBackColor().getRGB());
                        config.setInteger("mass.focuscolor",
                                getMainPanel().getMassColor().getFocusBackColor().getRGB());
                        config.setInteger("mass.focuscolor2",
                                getMainPanel().getMassColor().getFocusBackColor2().getRGB());
                        config.setInteger("mass.fontsize",
                                getJsFontSize().getValue());
                        config.set("window.lastdir",
                                getLastDirectory());
                        config.saveXML();
                    } catch (Exception error) {
                        error.printStackTrace();
                    }
                    System.exit(0);
                }
            });
        }
        return miExit;
    }

    private JMenuItem getMiStart() {
        if (miStart == null) {
            miStart = new JMenuItem("Jn");
            miStart.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getMainPanel().clear();
                    getScorePanel().score();
                    getTimePanel().clear();
                    getTimePanel().start();
                    getMainPanel().setFocus();
                }
            });
        }
        return miStart;
    }

    private JMenuItem getMiCheck() {
        if (miCheck == null) {
            miCheck = new JMenuItem("̓_");
            miCheck.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getMainPanel().check();
                    getScorePanel().score();
                    getTimePanel().stop();
                }
            });
        }
        return miCheck;
    }

    private JMenuItem getMiClear() {
        if (miClear == null) {
            miClear = new JMenuItem("");
            miClear.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getMainPanel().clear();
                    getScorePanel().score();
                    getTimePanel().clear();
                }
            });
        }
        return miClear;
    }

    private JMenuItem getMiOption() {
        if (miOption == null) {
            miOption = new JMenuItem("ݒ");
            miOption.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OptionDialog dialog =
                        new OptionDialog(MainFrame.this, getMainPanel());
                    dialog.setVisible(true);
                }
            });
        }
        return miOption;
    }

    private JMenuItem getMiVersion() {
        if (miVersion == null) {
            miVersion = new JMenuItem("o[W");
            miVersion.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JOptionPane.showMessageDialog(MainFrame.this, 
                        XXMath + "\n"
                      + "Copyright (C) 2007 Ryo Ikeda. All rights reserved.",
                        "o[W", JOptionPane.INFORMATION_MESSAGE);
                }
            });
        }
        return miVersion;
    }

    private String getLastDirectory() {
        return lastDir.toString();
    }

    private void setLastDirector(String path) {
        if (path == null) {
            throw new NullPointerException();
        }
        lastDir = new File(path);
    }

    /**
     * I΂ꂽbNAhtB[ɕύX郊Xi[B
     */
    private class LookAndFeelActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JRadioButtonMenuItem mi = (JRadioButtonMenuItem)e.getSource();
            try {
                String name = getLookAndFeelClassName(mi.getText());
                UIManager.setLookAndFeel(name);
                lookAndFeel = name;
                SwingUtilities.updateComponentTreeUI(MainFrame.this);
            } catch (Exception error) {
                error.printStackTrace();
            }
        }
    }

    /**
     * CSVt@Cpt@CtB^[B
     */
    private static class CSVFileFilter extends FileFilter {
        @Override
        public boolean accept(File file) {
            return (file.isDirectory() || file.getName().endsWith(".csv"));
        }
        @Override
        public String getDescription() {
            return "eLXgCSV (*.csv)";
        }
    }

}
