package xxmath.frame;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import xxmath.hundred.Hundred;
import xxmath.hundred.HundredBuilder;
import xxmath.util.DigitKeyListener;

/**
 * S܂vZ\郁CplB
 * 
 * @author Ryo IKEDA
 */
public class MainPanel extends JPanel {

    /** }X̔wiFݒIuWFNg */
    private final MassColor massColor = new MassColor();

    /** tH[JXړ[hpNXIuWFNg */
    public final MoveMode MOVE_NONE = new NullMove();
    public final MoveMode MOVE_RIGHT = new RightMove();
    public final MoveMode MOVE_DOWN = new DownMove();
    public final MoveMode MOVE_RANDOM = new RandomMove();

    /** tH[JXړ[h */
    private MoveMode move = MOVE_RIGHT;

    /** tH[JXړ܂ł̎(~b) */
    private int moveInterval = 100;

    /** 㕔ɕ\郂[hx */
    private JLabel lbMode;
    
    /** ̐p̃xz */
    private JLabel[] lbX;

    /** ̐p̃xz */
    private JLabel[] lbY;

    /** ̓tB[h̔z */
    private Mass[][] mass;

    /** S܂vZIuWFNg */
    private Hundred hundred;

    /** ̓_ۂ̓_ */
    private int score = 0;
    
    /** ̃tHgTCY */
    private int fontSize = 16;
    
    /** Xi[ */
    private FocusListener tfFocusListener = new TfFocusListener();
    private KeyListener tfKeyListener = new TfKeyListener();
    private DocumentListener tfDocumentListener = new TfDocumentListener();

    /**
     * ftHgRXgN^B
     */
    public MainPanel() {
        this(HundredBuilder.createAddition(10, 0, 9, 0, 9));
    }

    /**
     * RXgN^BS܂vZZbgB
     * 
     * @param h S܂vZIuWFNg
     * @throws NullPointerException null̏ꍇ
     */
    public MainPanel(Hundred h) {
        setHundred(h);
    }

    /**
     * S܂vZZbgp̃xƃeLXgtB[h쐬B
     * 
     * @param h S܂vZ̃IuWFNg
     * @throws NullPointerException null̏ꍇ
     */
    public void setHundred(Hundred h) {
        if (h == null) {
            throw new NullPointerException();
        }
        hundred = h;
        if (mass == null || mass.length != h.getSize()) {
            removeAll();
            create();
            validate();
            repaint();
        } else {
            update();
        }
    }

    /**
     * S܂vZIuWFNgԂB
     * 
     * @return ݂̕S܂vZIuWFNg
     */
    public Hundred getHundred() {
        return hundred;
    }

    /**
     * eLXgtB[hɂwiFftHgɖ߂B
     */
    public void clear() {
        score = 0;
        for (int i = 0; i < mass.length; i++) {
            for (int j = 0; j < mass[0].length; j++) {
                mass[i][j].setText("");
                mass[i][j].setBackground(massColor.getDefaultBackColor());
            }
        }
    }

    /**
     * eLXgtB[h̓_BƕsF\B
     */
    public void check() {
        score = 0;
        for (int i = 0; i < mass.length; i++) {
            for (int j = 0; j < mass[0].length; j++) {
                try {
                    int input = Integer.parseInt(mass[i][j].getText());
                    int answer = getHundred().getAnswersAt(
                                mass[i][j].getIdX(), mass[i][j].getIdY());
                    if (input == answer) {
                        score++;
                        mass[i][j].setBackground(massColor.getCorrectBackColor());
                    } else {
                        mass[i][j].setBackground(massColor.getWrongBackColor());
                    }
                } catch (NumberFormatException ignore) {
                }
            }
        }
    }

    /**
     * eLXgtB[hɃtH[JXZbgB
     */
    public void setFocus() {
        move.setFocus();
    }

    /**
     * eLXgtB[h̑ԂB
     */
    public int getTotal() {
        return (mass.length * mass[0].length);
    }

    /**
     * eLXgtB[h̐𐔂ԂB
     */
    public int getScore() {
        return score;
    }

    /**
     * ݂̈ړ[hݒԂB
     */
    public Object getMove() {
        return move;
    }

    /**
     * ړ[hݒ肷B
     * MainPanel.MOVE_NONE / MOVE_RIGHT / MOVE_DOWN / MOVE_RANDOM
     * ̂ꂩB
     */
    public void setMove(Object m) {
        if (m == null) {
            throw new NullPointerException();
        }
        move = (MoveMode)m;
    }

    /**
     * }X̐FݒIuWFNgԂB
     * 
     * @return FݒIuWFNg
     */
    public MassColor getMassColor() {
        return massColor;
    }

    /**
     * tH[JXړ܂ł̎ԂԂB
     */
    public int getMoveInterval() {
        return moveInterval;
    }

    /**
     * tH[JXړ܂ł̎Ԃݒ肷B
     */
    public void setMoveInterval(int time) {
        if (time < 0) {
            throw new IllegalArgumentException();
        }
        moveInterval = time;
    }

    /**
     * tHgTCYݒ肷B
     */
    public void setFontSize(int size) {
        fontSize = size;
        Font font = new Font("San Serif", Font.PLAIN, fontSize);
        lbMode.setFont(font);
        for (int i = 0; i < mass.length; i++) {
            lbX[i].setFont(font);
            lbY[i].setFont(font);
            for (int j = 0; j < mass[0].length; j++) {
                mass[i][j].setFont(font);
            }
        }
    }

    /**
     * S܂vZp̃xƃeLXgtB[h쐬B
     */
    private void create() {
        lbMode = new JLabel(getHundred().getSymbol(), JLabel.CENTER);
        add(lbMode);

        lbX = new JLabel[getHundred().getSize()];
        lbY = new JLabel[getHundred().getSize()];
        mass = new Mass[getHundred().getSize()][getHundred().getSize()];

        /*
         * ̐x̍쐬Aǉ
         */
        for (int i = 0; i < lbX.length; i++) {
            String text = String.valueOf(getHundred().getXAt(i));
            lbX[i] = new JLabel(text, JLabel.CENTER);
            add(lbX[i]);
        }

        /*
         * ̐Ɠ̓tB[h̍쐬Aǉ
         */
        for (int i = 0; i < mass.length; i++) {
            String text = String.valueOf(getHundred().getYAt(i));
            lbY[i] = new JLabel(text, JLabel.CENTER);
            add(lbY[i]);

            for (int j = 0; j < mass[0].length; j++) {
                mass[i][j] = new Mass(j, i);
                mass[i][j].addFocusListener(tfFocusListener);
                mass[i][j].addKeyListener(tfKeyListener);
                mass[i][j].getDocument()
                    .addDocumentListener(tfDocumentListener);
                add(mass[i][j]);
            }
        }
        setFontSize(fontSize);
        setLayout(new GridLayout(mass.length + 1, mass[0].length + 1));
    }

    /**
     * ςBS܂vZ̃TCYꍇ͂gB
     */
    private void update() {
        clear();
        lbMode.setText(getHundred().getSymbol());
        for (int i = 0; i < lbX.length; i++) {
            String textX = String.valueOf(getHundred().getXAt(i));
            lbX[i].setText(textX);
            String textY = String.valueOf(getHundred().getYAt(i));
            lbY[i].setText(textY);
        }
    }

    /**
     * ݃tH[JXĂ}XԂB
     * tH[JXȂnullԂB
     */
    private Mass getCurrentMass() {
        for (int i = 0; i < mass.length; i++) {
            for (int j = 0; j < mass[0].length; j++) {
                if (mass[i][j].hasFocus()) {
                    return mass[i][j];
                }
            }
        }
        return null;
    }

    /**
     * Ẽ}XɈړB
     */
    private void grabNextFocus() {
        Mass ms = getCurrentMass();
        if (ms == null) {
            return;
        }
        int col = ms.getIdX();
        int row = ms.getIdY();
        int last = mass.length - 1;
        if (col != last) {
            mass[row][++col].grabFocus();
        } else if (row != last) {
            mass[++row][0].grabFocus();
        } else {
            mass[0][0].grabFocus();
        }
    }

    /**
     * ̃}XɈړB
     */
    private void grabPreviousFocus() {
        Mass ms = getCurrentMass();
        if (ms == null) {
            return;
        }
        int col = ms.getIdX();
        int row = ms.getIdY();
        int last = mass.length - 1;
        if (col != 0) {
            mass[row][--col].grabFocus();
        } else if (row != 0) {
            mass[--row][last].grabFocus();
        } else {
            mass[last][last].grabFocus();
        }
    }
    
    /**
     * ̃}XɈړB
     */
    private void grabUpperFocus() {
        Mass ms = getCurrentMass();
        if (ms == null) {
            return;
        }
        int col = ms.getIdX();
        int row = ms.getIdY();
        int last = mass.length - 1;
        if (row != 0) {
            mass[--row][col].grabFocus();
        } else if (col != 0) {
            mass[last][--col].grabFocus();
        } else {
            mass[last][last].grabFocus();
        }
    }

    /**
     * ̃}XɈړB
     */
    private void grabLowerFocus() {
        Mass ms = getCurrentMass();
        if (ms == null) {
            return;
        }
        int col = ms.getIdX();
        int row = ms.getIdY();
        int last = mass.length - 1;
        if (row != last) {
            mass[++row][col].grabFocus();
        } else if (col != last) {
            mass[0][++col].grabFocus();
        } else {
            mass[0][0].grabFocus();
        }
    }

    /**
     * tH[JX̂}XƓsE̖Ƀ{[_[B
     */
    private class TfFocusListener implements FocusListener {
        public void focusGained(FocusEvent e) {
            Mass ms = (Mass)e.getSource();
            int x = ms.getIdX();
            int y = ms.getIdY();
            Border border = BorderFactory.createLineBorder(massColor.getFocusBorderColor(), 2);

            lbX[x].setBorder(border);
            lbY[y].setBorder(border);
            
            for (int i = 0; i < mass.length; i++) {
                if (i != x) {
                    mass[y][i].setBackground(massColor.getFocusBackColor2());
                }
                if (i != y) {
                    mass[i][x].setBackground(massColor.getFocusBackColor2());
                }
            }
        }
        public void focusLost(FocusEvent e) {
            Mass ms = (Mass)e.getSource();
            int x = ms.getIdX();
            int y = ms.getIdY();

            lbX[x].setBorder(null);
            lbY[y].setBorder(null);

            for (int i = 0; i < mass.length; i++) {
                mass[y][i].setBeforeBackground();
                mass[i][x].setBeforeBackground();
            }
        }
    }

    /**
     * ㉺EL[Ń}XړB
     */
    private class TfKeyListener extends KeyAdapter {
        @Override
        public void keyPressed(KeyEvent e) {
            int mod = e.getModifiersEx();
            if (mod == 0) {
                switch (e.getKeyCode()) {
                case KeyEvent.VK_UP:
                    grabUpperFocus();
                    break;
                case KeyEvent.VK_DOWN:
                    grabLowerFocus();
                    break;
                case KeyEvent.VK_LEFT:
                    grabPreviousFocus();
                    break;
                case KeyEvent.VK_RIGHT:
                    grabNextFocus();
                    break;
                }
            }
        }
    }

    /**
     * ͂̕ꂽ玟̃}X֎ړB
     */
    private class TfDocumentListener implements DocumentListener, Runnable {
        public void insertUpdate(DocumentEvent e) {
            Mass ms = getCurrentMass();
            if (ms == null) {
                return;
            }
            try {
                Integer.parseInt(ms.getText());

                int x = ms.getIdX();
                int y = ms.getIdY();
                int answer = getHundred().getAnswersAt(x, y);
                int anslength = String.valueOf(answer).length();
                if (anslength == ms.getText().length()) {
                    new Thread(this).start();
                }
            } catch (NumberFormatException ignore) {
            }
        }
        public void changedUpdate(DocumentEvent e) {}
        public void removeUpdate(DocumentEvent e) {}

        public void run() {
            try {
                Thread.sleep(getMoveInterval());
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        move.next();
                    }
                });
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * tH[JXړC^[tFCXB
     */
    private interface MoveMode {
        /** ŏ̃}XɃtH[JXZbgB */
        public void setFocus();
        /** ̃}XփtH[JXړB */
        public void next();
    }

    /**
     * ȂB
     */
    private class NullMove implements MoveMode {
        public void next() {}
        public void setFocus() {}
    }

    /**
     * Ẽ}XֈړB
     */
    private class RightMove implements MoveMode {
        public void next() {
            Mass mass = getCurrentMass();
            if (mass != null && !mass.isLastMass()) {
                grabNextFocus();
            }
        }
        public void setFocus() {
            mass[0][0].grabFocus();
        }
    }

    /**
     * ̃}XֈړB
     */
    private class DownMove implements MoveMode {
        public void next() {
            Mass mass = getCurrentMass();
            if (mass != null && !mass.isLastMass()) {
                grabLowerFocus();
            }
        }
        public void setFocus() {
            mass[0][0].grabFocus();
        }
    }

    /**
     * _ɈړB
     */
    private class RandomMove implements MoveMode {
        private final Random random = new Random();

        public void next() {
            ArrayList<Mass> list = new ArrayList<Mass>();
            while (list.size() < getTotal()) {
                int x = random.nextInt(mass[0].length);
                int y = random.nextInt(mass.length);

                String text = mass[y][x].getText();
                if (text.matches("")) {
                    mass[y][x].grabFocus();
                    return;
                } else if (!list.contains(mass[y][x])) {
                    list.add(mass[y][x]);
                }
            }
        }
        public void setFocus() {
            next();
        }
    }

    /**
     * S܂vZ̃}X\NXB
     */
    private class Mass extends JTextField {

        private FocusListener tfFocusListener = new MassFocusListener();
        private KeyListener tfKeyListener = new MassKeyListener();
        private DocumentListener tfDocumentListener = new MassDocumentListener();

        private int idx;
        private int idy;

        private Color formerBackColor;

        /**
         * RXgN^Bgǂɂ邩̏B
         * 
         * @param idx XCfbNX
         * @param idy YCfbNX
         */
        Mass(int x, int y) {
            idx = x;
            idy = y;
            setHorizontalAlignment(CENTER);
            setBackground(massColor.getDefaultBackColor());
            setSelectedTextColor(Color.WHITE);
            setSelectionColor(Color.BLACK);
            addFocusListener(tfFocusListener);
            addKeyListener(tfKeyListener);
            getDocument().addDocumentListener(tfDocumentListener);
        }

        /**
         * XCfbNXԂB
         */
        int getIdX() {
            return idx;
        }

        /**
         * YCfbNXԂB
         */
        int getIdY() {
            return idy;
        }

        /**
         * ݂̔wiFLāAVwiFݒ肷B
         */
        @Override
        public void setBackground(Color color) {
            formerBackColor = this.getBackground();
            super.setBackground(color);
        }

        /**
         * ȑOɋLwiFɖ߂B
         */
        void setBeforeBackground() {
            super.setBackground(formerBackColor);
        }

        /**
         * }XŌ̃}XǂԂB
         */
        boolean isLastMass() {
            return (getIdX() == getHundred().getSize() - 1
                    && getIdY() == getHundred().getSize() - 1);
        }

        /**
         * tH[JXɂĔwiFςB
         */
        private class MassFocusListener implements FocusListener {
            public void focusGained(FocusEvent e) {
                Mass ms = (Mass)e.getSource();
                ms.setBackground(massColor.getFocusBackColor());
                ms.selectAll();
            }
            public void focusLost(FocusEvent e) {
                Mass ms = (Mass)e.getSource();
                ms.setBeforeBackground();
            }
        }

        /**
         * ͂ȂꂽꍇA
         * tH[JXOꂽƂ̔wiFftHgɖ߂B
         */
        private class MassDocumentListener implements DocumentListener {
            public void changedUpdate(DocumentEvent e) {}
            public void insertUpdate(DocumentEvent e) {
                formerBackColor = massColor.getDefaultBackColor();
            }
            public void removeUpdate(DocumentEvent e) {
                formerBackColor = massColor.getDefaultBackColor();
            }
        }

        /**
         * ̌ȏ̓́AȊO̓͂󂯕tȂB
         */
        private class MassKeyListener extends DigitKeyListener {
            @Override
            public void keyTyped(KeyEvent e) {
                Mass mass = (Mass)e.getSource();
                this.setMaxDigit(getAnswerLength(mass));
                super.keyTyped(e);
            }

            private int getAnswerLength(Mass mass) {
                int x = mass.getIdX();
                int y = mass.getIdY();
                int answer = getHundred().getAnswersAt(x, y);
                return String.valueOf(answer).length();
            }
        }

    }

}
