package xxmath.frame;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import xxmath.util.DigitKeyListener;

/**
 * ݒ_CAONXB
 * 
 * @author Ryo IKEDA
 */
public class OptionDialog extends JDialog {

    private MainPanel pnMain;
    private JPanel jContentPane;
    private JPanel pnTime;
    private JLabel lbTime;
    private JTextField tfTime;
    private JPanel pnMode;
    private ButtonGroup bgModes;
    private JPanel pnButton;
    private JButton btOK;
    private JButton btCansel;
    private JPanel pnMove;
    private JRadioButton rbNone;
    private JRadioButton rbRight;
    private JRadioButton rbDown;
    private JRadioButton rbRandom;
    private JTabbedPane jTabbedPane;
    private JPanel pnColor;
    private JLabel lbDefault;
    private JLabel lbActive;
    private JLabel lbActive2;
    private JLabel lbCorrect;
    private JLabel lbWrong;
    private JLabel lbBorder;
    private JButton btDefault;
    private JButton btActive;
    private JButton btActive2;
    private JButton btCorrect;
    private JButton btWrong;
    private JButton btBorder;

    public OptionDialog(MainFrame frame, MainPanel panel) {
        super(frame, "ݒ", true);
        pnMain = panel;
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        initialize();
    }

    private void initialize() {
        setContentPane(getJContentPane());
        setLocationToCenter();

        addWindowListener(new WindowAdapter() {
            @Override
            public void windowOpened(WindowEvent e) {
                String text = String.valueOf(pnMain.getMoveInterval());
                getTfTime().setText(text);

                Object move = pnMain.getMove();
                if (move == pnMain.MOVE_RIGHT) {
                    getRbRight().setSelected(true);
                } else if (move == pnMain.MOVE_DOWN) {
                    getRbDown().setSelected(true);
                } else if (move == pnMain.MOVE_RANDOM) {
                    getRbRandom().setSelected(true);
                } else if (move == pnMain.MOVE_NONE) {
                    getRbNone().setSelected(true);
                }

                getLbDefault().setBackground(pnMain.getMassColor().getDefaultBackColor());
                getLbActive().setBackground(pnMain.getMassColor().getFocusBackColor());
                getLbActive2().setBackground(pnMain.getMassColor().getFocusBackColor2());
                getLbCorrect().setBackground(pnMain.getMassColor().getCorrectBackColor());
                getLbWrong().setBackground(pnMain.getMassColor().getWrongBackColor());
                getLbBorder().setBorder(BorderFactory.createLineBorder(
                                        pnMain.getMassColor().getFocusBorderColor(), 2));
                }
            @Override
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
    }

    private void setLocationToCenter() {
        pack();
        int x = getOwner().getX() + (getOwner().getWidth() - getWidth()) / 2;
        int y = getOwner().getY() + (getOwner().getHeight() - getHeight()) / 2;
        setBounds(x, y, getWidth(), getHeight());
    }

    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel(new BorderLayout());
            jContentPane.add(getJTabbedPane(), BorderLayout.CENTER);
            jContentPane.add(getPnButton(), BorderLayout.SOUTH);
        }
        return jContentPane;
    }

    private JPanel getPnMove() {
        if (pnMove == null) {
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 1;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            pnMove = new JPanel();
            pnMove.setLayout(new GridBagLayout());
            pnMove.add(getPnTime(), gridBagConstraints);
            pnMove.add(getPnMode(), gridBagConstraints1);
        }
        return pnMove;
    }

    private JPanel getPnTime() {
        if (pnTime == null) {
            pnTime = new JPanel();
            pnTime.add(getLbTime());
            pnTime.add(getTfTime());
        }
        return pnTime;
    }

    private JPanel getPnMode() {
        if (pnMode == null) {
            pnMode = new JPanel();
            pnMode.setLayout(new GridLayout(4, 1));
            pnMode.add(getRbRight());
            pnMode.add(getRbDown());
            pnMode.add(getRbRandom());
            pnMode.add(getRbNone());
            bgModes = new ButtonGroup();
            bgModes.add(getRbNone());
            bgModes.add(getRbRight());
            bgModes.add(getRbDown());
            bgModes.add(getRbRandom());
        }
        return pnMode;
    }

    private JPanel getPnButton() {
        if (pnButton == null) {
            pnButton = new JPanel();
            pnButton.add(getBtOK());
            pnButton.add(getBtCansel());
            getRootPane().setDefaultButton(getBtOK());
        }
        return pnButton;
    }

    private JLabel getLbTime() {
        if (lbTime == null) {
            lbTime = new JLabel("ړ(~b)", JLabel.CENTER);
        }
        return lbTime;
    }

    private JTextField getTfTime() {
        if (tfTime == null) {
            tfTime = new JTextField(3);
            tfTime.setHorizontalAlignment(JTextField.RIGHT);
            tfTime.addKeyListener(new DigitKeyListener(3, false));
            tfTime.addFocusListener(new FocusAdapter() {
                @Override
                public void focusGained(FocusEvent e) {
                    tfTime.selectAll();
                }
            });
            tfTime.getDocument().addDocumentListener(new DocumentListener() {
                public void changedUpdate(DocumentEvent e) {}
                public void insertUpdate(DocumentEvent e) {
                    setButton();
                }
                public void removeUpdate(DocumentEvent e) {
                    setButton();
                }
                private void setButton() {
                    if (tfTime.getText().matches("\\d{1,3}")) {
                        getBtOK().setEnabled(true);
                    } else {
                        getBtOK().setEnabled(false);
                    }
                }
            });
        }
        return tfTime;
    }

    private JButton getBtOK() {
        if (btOK == null) {
            btOK = new JButton("OK");
            btOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int time = Integer.parseInt(getTfTime().getText());
                    pnMain.setMoveInterval(time);
                    if (getRbRight().isSelected()) {
                        pnMain.setMove(pnMain.MOVE_RIGHT);
                    } else if (getRbDown().isSelected()) {
                        pnMain.setMove(pnMain.MOVE_DOWN);
                    } else if (getRbRandom().isSelected()) {
                        pnMain.setMove(pnMain.MOVE_RANDOM);
                    } else if (getRbNone().isSelected()) {
                        pnMain.setMove(pnMain.MOVE_NONE);
                    }
                    LineBorder border = (LineBorder)getLbBorder().getBorder();
                    pnMain.getMassColor().setFocusBorderColor(border.getLineColor());
                    pnMain.getMassColor().setCorrectBackColor(getLbCorrect().getBackground());
                    pnMain.getMassColor().setWrongBackColor(getLbWrong().getBackground());
                    pnMain.getMassColor().setDefaultBackColor(getLbDefault().getBackground());
                    pnMain.getMassColor().setFocusBackColor(getLbActive().getBackground());
                    pnMain.getMassColor().setFocusBackColor2(getLbActive2().getBackground());

                    dispose();
                }
            });
        }
        return btOK;
    }

    private JButton getBtCansel() {
        if (btCansel == null) {
            btCansel = new JButton("LZ");
            btCansel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            });
        }
        return btCansel;
    }

    private JRadioButton getRbNone() {
        if (rbNone == null) {
            rbNone = new JRadioButton("Ȃ");
        }
        return rbNone;
    }

    private JRadioButton getRbRight() {
        if (rbRight == null) {
            rbRight = new JRadioButton("E");
        }
        return rbRight;
    }

    private JRadioButton getRbDown() {
        if (rbDown == null) {
            rbDown = new JRadioButton("");
        }
        return rbDown;
    }

    private JRadioButton getRbRandom() {
        if (rbRandom == null) {
            rbRandom = new JRadioButton("_");
        }
        return rbRandom;
    }

    private JTabbedPane getJTabbedPane() {
        if (jTabbedPane == null) {
            jTabbedPane = new JTabbedPane();
            jTabbedPane.addTab("ړ", getPnMove());
            jTabbedPane.addTab("F", getPnColor());
            jTabbedPane.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    pack();
                }
            });
        }
        return jTabbedPane;
    }

    private JPanel getPnColor() {
        if (pnColor == null) {
            GridLayout gridLayout = new GridLayout();
            gridLayout.setRows(6);
            gridLayout.setColumns(2);
            pnColor = new JPanel();
            pnColor.setLayout(gridLayout);
            pnColor.add(getLbDefault(), null);
            pnColor.add(getBtDefault(), null);
            pnColor.add(getLbActive(), null);
            pnColor.add(getBtActive(), null);
            pnColor.add(getLbActive2(), null);
            pnColor.add(getBtActive2(), null);
            pnColor.add(getLbCorrect(), null);
            pnColor.add(getBtCorrect(), null);
            pnColor.add(getLbWrong(), null);
            pnColor.add(getBtWrong(), null);
            pnColor.add(getLbBorder(), null);
            pnColor.add(getBtBorder(), null);
        }
        return pnColor;
    }

    private JLabel getLbBorder() {
        if (lbBorder == null) {
            lbBorder = new JLabel("g");
            lbBorder.setHorizontalAlignment(SwingConstants.CENTER);
            lbBorder.setOpaque(true);
        }
        return lbBorder;
    }

    private JLabel getLbDefault() {
        if (lbDefault == null) {
            lbDefault = new JLabel("ftHg");
            lbDefault.setHorizontalAlignment(SwingConstants.CENTER);
            lbDefault.setOpaque(true);
        }
        return lbDefault;
    }

    private JLabel getLbActive() {
        if (lbActive == null) {
            lbActive = new JLabel("I}X");
            lbActive.setHorizontalAlignment(SwingConstants.CENTER);
            lbActive.setOpaque(true);
        }
        return lbActive;
    }

    private JLabel getLbActive2() {
        if (lbActive2 == null) {
            lbActive2 = new JLabel("Is");
            lbActive2.setHorizontalAlignment(SwingConstants.CENTER);
            lbActive2.setOpaque(true);
        }
        return lbActive2;
    }

    private JLabel getLbCorrect() {
        if (lbCorrect == null) {
            lbCorrect = new JLabel("");
            lbCorrect.setHorizontalAlignment(SwingConstants.CENTER);
            lbCorrect.setOpaque(true);
        }
        return lbCorrect;
    }

    private JLabel getLbWrong() {
        if (lbWrong == null) {
            lbWrong = new JLabel("s");
            lbWrong.setHorizontalAlignment(SwingConstants.CENTER);
            lbWrong.setOpaque(true);
        }
        return lbWrong;
    }

    private JButton getBtDefault() {
        if (btDefault == null) {
            btDefault = new JButton("F");
            btDefault.addActionListener(new ColorActionListener(getLbDefault()));
        }
        return btDefault;
    }

    private JButton getBtActive() {
        if (btActive == null) {
            btActive = new JButton("F");
            btActive.addActionListener(new ColorActionListener(getLbActive()));
        }
        return btActive;
    }

    private JButton getBtActive2() {
        if (btActive2 == null) {
            btActive2 = new JButton("F");
            btActive2.addActionListener(new ColorActionListener(getLbActive2()));
        }
        return btActive2;
    }

    private JButton getBtCorrect() {
        if (btCorrect == null) {
            btCorrect = new JButton("F");
            btCorrect.addActionListener(new ColorActionListener(getLbCorrect()));
        }
        return btCorrect;
    }

    private JButton getBtWrong() {
        if (btWrong == null) {
            btWrong = new JButton("F");
            btWrong.addActionListener(new ColorActionListener(getLbWrong()));
        }
        return btWrong;
    }

    private JButton getBtBorder() {
        if (btBorder == null) {
            btBorder = new JButton("F");
            btBorder.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    LineBorder border = (LineBorder)getLbBorder().getBorder();
                    Color color = JColorChooser.showDialog(OptionDialog.this,
                            "FI", border.getLineColor());
                    if (color != null) {
                        getLbBorder().setBorder(
                                BorderFactory.createLineBorder(color, 2));
                    }
                }
            });
        }
        return btBorder;
    }

    /**
     * J[`[U[őI񂾐Fxɐݒ肷ANVXi[B
     */
    private class ColorActionListener implements ActionListener {

        private JLabel label;

        public ColorActionListener(JLabel lb) {
            if (lb == null) {
                throw new NullPointerException();
            }
            label = lb;
        }

        public void actionPerformed(ActionEvent e) {
            Color color = JColorChooser.showDialog(OptionDialog.this,
                    "FI", label.getBackground());
            if (color != null) {
                label.setBackground(color);
            }
        }

    }

}
